@@ -5,8 +5,8 @@
 ;; Author: Jethro Kuan <jethrokuan95@gmail.com>
 ;; URL: https://github.com/org-roam/org-roam
 ;; Keywords: org-mode, roam, convenience
-;; Version: 1.2.1
-;; Package-Requires: ((emacs "26.1") (dash "2.13") (f "0.17.2") (s "1.12.0") (org "9.3") (emacsql "3.0.0") (emacsql-sqlite3 "1.0.0"))
+;; Version: 1.2.3
+;; Package-Requires: ((emacs "26.1") (dash "2.13") (f "0.17.2") (s "1.12.0") (org "9.3") (emacsql "3.0.0") (emacsql-sqlite3 "1.0.2"))
 
 ;; This file is NOT part of GNU Emacs.
 
@@ -31,6 +31,7 @@
 ;;; Code:
 ;;;; Library Requires
 (require 'org-capture)
+(require 'org-roam-macs)
 (require 'dash)
 (require 's)
 (require 'cl-lib)
@@ -40,20 +41,15 @@
 (defvar org-roam-directory)
 (defvar org-roam-mode)
 (defvar org-roam-title-to-slug-function)
+
 (declare-function  org-roam--get-title-path-completions "org-roam")
 (declare-function  org-roam--get-ref-path-completions   "org-roam")
 (declare-function  org-roam--file-path-from-id          "org-roam")
 (declare-function  org-roam--find-file                  "org-roam")
-(declare-function  org-roam--format-link                "org-roam")
+(declare-function  org-roam-format-link                "org-roam")
 (declare-function  org-roam-mode                        "org-roam")
 (declare-function  org-roam-completion--completing-read "org-roam-completion")
 
-(defvar org-roam-capture--file-name-default "%<%Y%m%d%H%M%S>"
-  "The default file name format for Org-roam templates.")
-
-(defvar org-roam-capture--header-default "#+title: ${title}\n"
-  "The default capture header for Org-roam templates.")
-
 (defvar org-roam-capture--file-path nil
   "The file path for the Org-roam capture.
 This variable is set during the Org-roam capture process.")
@@ -79,11 +75,11 @@ note with the given `ref'.")
 (defvar org-roam-capture-additional-template-props nil
   "Additional props to be added to the Org-roam template.")
 
-(defconst org-roam-capture--template-keywords '(:file-name :head)
+(defconst org-roam-capture--template-keywords '(:file-name :head :olp)
   "Keywords used in `org-roam-capture-templates' specific to Org-roam.")
 
 (defcustom org-roam-capture-templates
-  '(("d" "default" plain (function org-roam-capture--get-point)
+  `(("d" "default" plain (function org-roam-capture--get-point)
      "%?"
      :file-name "%<%Y%m%d%H%M%S>-${slug}"
      :head "#+title: ${title}\n"
@@ -102,7 +98,7 @@ The Org-roam capture-templates  builds on the default behaviours of
 
 3. The `:head' key is added, which contains the template that is
    inserted upon the creation of a new file. This is where you
-   should add your note metadata should go.
+   your note metadata should go.
 
 Each template should have the following structure:
 
@@ -223,10 +219,10 @@ Template string   :\n%v")
                  ((const :format "%v " :kill-buffer) (const t))))))
 
 (defcustom org-roam-capture-ref-templates
-  '(("r" "ref" plain (function org-roam-capture--get-point)
+  '(("r" "ref" plain #'org-roam-capture--get-point
      "%?"
      :file-name "${slug}"
-     :head "#+title: ${title}\n#+roam_key: ${ref}\n"
+     :head "#+title: ${title}\n#+roam_key: ${ref}"
      :unnarrowed t))
   "The Org-roam templates used during a capture from the roam-ref protocol.
 Details on how to specify for the template is given in `org-roam-capture-templates'."
@@ -320,26 +316,36 @@ the capture)."
 
 (defun org-roam-capture--finalize ()
   "Finalize the `org-roam-capture' process."
-  (unless org-note-abort
-    (pcase (org-roam-capture--get :finalize)
-      ('find-file
-       (when-let ((file-path (org-roam-capture--get :file-path)))
-         (org-roam--find-file file-path)
-         (run-hooks 'org-roam-capture-after-find-file-hook)))
-      ('insert-link
-       (when-let* ((mkr (org-roam-capture--get :insert-at))
-                   (buf (marker-buffer mkr)))
-         (with-current-buffer buf
-           (when-let ((region (org-roam-capture--get :region))) ;; Remove previously selected text.
-             (delete-region (car region) (cdr region)))
-           (let ((path (org-roam-capture--get :file-path))
-                 (desc (org-roam-capture--get :link-description)))
-             (if (eq (point) (marker-position mkr))
-                 (insert (org-roam--format-link path desc))
-               (org-with-point-at mkr
-                 (insert (org-roam--format-link path desc))))))))))
-  (org-roam-capture--save-file-maybe)
-  (remove-hook 'org-capture-after-finalize-hook #'org-roam-capture--finalize))
+  (let* ((finalize (org-roam-capture--get :finalize))
+         ;; In case any regions were shielded before, unshield them
+         (region (when-let ((region (org-roam-capture--get :region)))
+                   (org-roam-unshield-region (car region) (cdr region))))
+         (beg (car region))
+         (end (cdr region)))
+    (unless org-note-abort
+      (pcase finalize
+        ('find-file
+         (when-let ((file-path (org-roam-capture--get :file-path)))
+           (org-roam--find-file file-path)
+           (run-hooks 'org-roam-capture-after-find-file-hook)))
+        ('insert-link
+         (when-let* ((mkr (org-roam-capture--get :insert-at))
+                     (buf (marker-buffer mkr)))
+           (with-current-buffer buf
+             (when region
+               (delete-region (car region) (cdr region)))
+             (let ((path (org-roam-capture--get :file-path))
+                   (type (org-roam-capture--get :link-type))
+                   (desc (org-roam-capture--get :link-description)))
+               (if (eq (point) (marker-position mkr))
+                   (insert (org-roam-format-link path desc type))
+                 (org-with-point-at mkr
+                   (insert (org-roam-format-link path desc type))))))))))
+    (when region
+      (set-marker beg nil)
+      (set-marker end nil))
+    (org-roam-capture--save-file-maybe)
+    (remove-hook 'org-capture-after-finalize-hook #'org-roam-capture--finalize)))
 
 (defun org-roam-capture--install-finalize ()
   "Install `org-roam-capture--finalize' if the capture is an Org-roam capture."
@@ -404,27 +410,36 @@ aborted, we do the following:
 the file if the original value of :no-save is not t and
 `org-note-abort' is not t."
   (let* ((name-templ (or (org-roam-capture--get :file-name)
-                         org-roam-capture--file-name-default))
+                         (user-error "Template needs to specify `:file-name'")))
          (new-id (s-trim (org-roam-capture--fill-template
                           name-templ)))
          (file-path (org-roam--file-path-from-id new-id))
          (roam-head (or (org-roam-capture--get :head)
-                        org-roam-capture--header-default))
+                        ""))
          (org-template (org-capture-get :template))
          (roam-template (concat roam-head org-template)))
-    (unless (file-exists-p file-path)
+    (unless (or (file-exists-p file-path)
+                (cl-some (lambda (buffer)
+                           (string= (buffer-file-name buffer)
+                                    file-path))
+                         (buffer-list)))
       (make-directory (file-name-directory file-path) t)
       (org-roam-capture--put :orig-no-save (org-capture-get :no-save)
                              :new-file t)
-      (org-capture-put :template
-                       ;; Fixes org-capture-place-plain-text throwing 'invalid search bound'
-                       ;; when both :unnarowed t and "%?" is missing from the template string;
-                       ;; may become unnecessary when the upstream bug is fixed
-                       (if (s-contains-p "%?" roam-template)
-                           roam-template
-                         (concat roam-template "%?"))
-                       :type 'plain
-                       :no-save t))
+      (pcase org-roam-capture--context
+        ('dailies
+         ;; Populate the header of the daily file before capture to prevent it
+         ;; from appearing in the buffer-restriction
+         (save-window-excursion
+           (find-file file-path)
+           (insert (substring (org-capture-fill-template (concat roam-head "*"))
+                              0 -2))
+           (set-buffer-modified-p nil))
+         (org-capture-put :template org-template))
+        (_
+         (org-capture-put :template roam-template
+           :type 'plain)))
+      (org-capture-put :no-save t))
     file-path))
 
 (defun org-roam-capture--get-point ()
@@ -443,32 +458,53 @@ If there is no file with that ref, a file with that ref is created.
 
 This function is used solely in Org-roam's capture templates: see
 `org-roam-capture-templates'."
-  (let ((file-path (pcase org-roam-capture--context
-                     ('capture
-                      (or (cdr (assoc 'file org-roam-capture--info))
-                          (org-roam-capture--new-file)))
-                     ('title
-                      (org-roam-capture--new-file))
-                     ('dailies
-                      (org-capture-put :default-time (cdr (assoc 'time org-roam-capture--info)))
-                      (org-roam-capture--new-file))
-                     ('ref
-                      (let ((completions (org-roam--get-ref-path-completions))
-                            (ref (cdr (assoc 'ref org-roam-capture--info))))
-                        (if-let ((pl (cdr (assoc ref completions))))
-                            (plist-get pl :path)
-                          (org-roam-capture--new-file))))
-                     (_ (error "Invalid org-roam-capture-context")))))
+  (let* ((file-path (pcase org-roam-capture--context
+                      ('capture
+                       (or (cdr (assoc 'file org-roam-capture--info))
+                           (org-roam-capture--new-file)))
+                      ('title
+                       (org-roam-capture--new-file))
+                      ('dailies
+                       (org-capture-put :default-time (cdr (assoc 'time org-roam-capture--info)))
+                       (org-roam-capture--new-file))
+                      ('ref
+                       (let ((completions (org-roam--get-ref-path-completions))
+                             (ref (cdr (assoc 'ref org-roam-capture--info))))
+                         (if-let ((pl (cdr (assoc ref completions))))
+                             (plist-get pl :path)
+                           (org-roam-capture--new-file))))
+                      (_ (error "Invalid org-roam-capture-context")))))
     (org-capture-put :template
-                     (org-roam-capture--fill-template (org-capture-get :template)))
-    (org-roam-capture--put :file-path file-path)
+      (org-roam-capture--fill-template (org-capture-get :template)))
+    (org-roam-capture--put :file-path file-path
+                           :finalize (or (org-capture-get :finalize)
+                                         (org-roam-capture--get :finalize)))
     (while org-roam-capture-additional-template-props
       (let ((prop (pop org-roam-capture-additional-template-props))
             (val (pop org-roam-capture-additional-template-props)))
         (org-roam-capture--put prop val)))
     (set-buffer (org-capture-target-buffer file-path))
     (widen)
-    (goto-char (point-max))))
+    (if-let* ((olp (when (eq org-roam-capture--context 'dailies)
+                     (--> (org-roam-capture--get :olp)
+                          (pcase it
+                            ((pred stringp)
+                             (list it))
+                            ((pred listp)
+                             it)
+                            (wrong-type
+                             (signal 'wrong-type-argument
+                                     `((stringp listp)
+                                       ,wrong-type))))))))
+        (condition-case err
+            (when-let ((marker (org-find-olp `(,file-path ,@olp))))
+              (goto-char marker)
+              (set-marker marker nil))
+          (error
+           (when (org-roam-capture--get :new-file)
+             (kill-buffer))
+           (signal (car err) (cdr err))))
+      (goto-char (point-min)))))
 
 (defun org-roam-capture--convert-template (template)
   "Convert TEMPLATE from Org-roam syntax to `org-capture-templates' syntax."
@@ -483,6 +519,9 @@ This function is used solely in Org-roam's capture templates: see
          (let* ((key (pop rest))
                 (val (pop rest))
                 (custom (member key org-roam-capture--template-keywords)))
+           (when (and custom
+                      (not val))
+             (user-error "Invalid capture template format: %s\nkey %s cannot be nil" template key))
            (push val (if custom org-roam-plist options))
            (push key (if custom org-roam-plist options))))
        (append converted options `(:org-roam ,org-roam-plist))))
@@ -506,7 +545,8 @@ GOTO and KEYS argument have the same functionality as
 `org-capture'."
   (let* ((org-capture-templates (mapcar #'org-roam-capture--convert-template org-roam-capture-templates))
          (one-template-p (= (length org-capture-templates) 1))
-         org-capture-templates-contexts)
+         org-capture-templates-contexts
+         (org-capture-link-is-already-stored t))
     (when one-template-p
       (setq keys (caar org-capture-templates)))
     (if (or one-template-p
@@ -515,10 +555,11 @@ GOTO and KEYS argument have the same functionality as
       (funcall-interactively org-roam-capture-function))))
 
 ;;;###autoload
-(defun org-roam-capture ()
+(defun org-roam-capture (&optional goto keys)
   "Launches an `org-capture' process for a new or existing note.
-This uses the templates defined at `org-roam-capture-templates'."
-  (interactive)
+This uses the templates defined at `org-roam-capture-templates'.
+Arguments GOTO and KEYS see `org-capture'."
+  (interactive "P")
   (unless org-roam-mode (org-roam-mode))
   (let* ((completions (org-roam--get-title-path-completions))
          (title-with-keys (org-roam-completion--completing-read "File: "
@@ -531,7 +572,7 @@ This uses the templates defined at `org-roam-capture-templates'."
                                         (cons 'file file-path)))
           (org-roam-capture--context 'capture))
       (condition-case err
-          (org-roam-capture--capture)
+          (org-roam-capture--capture goto keys)
         (error (user-error "%s.  Please adjust `org-roam-capture-templates'"
                            (error-message-string err)))))))
 
